package ssw.annotation.AnnotationFileHandler;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/*
 * Created on 08.03.2005
 */

/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 * This class is the concrete implementation of an AnnotationFileHandler. It implements 
 * the AnnotatoinFileHandler interface and manages a number of annotations. The annotations 
 * are stored in a LinkedList- all types of AnnotationItems are allowed to be inserted. 
 * The global data is stored in a HashTable- both, key and value are assumed to be of 
 * the Java type String. 
 * This file is meant to be a container handling the annotations during runtime. 
 * It is possible to create an instance without indicating a file name. Annotations then can 
 * be added, and if one wants to save the file the name has to be passed. Another option is 
 * to generate an empty annotation file indicating the file name. If done so, one only has 
 * to use the save method without passing the file name. And the third option is to open an 
 * existing annotation file- which is done just like in option two, but the indicated file 
 * has to exist.
 * This architecture makes it possible to easily create, manage and rename files.
 * On saving, to the first line the count of heather elements (global data) is written. 
 * The following lines contain the heather elements. And after a blank line the annotations 
 * are written. The annotations are saved using the CSV string generated by the annotation 
 * objects. 
 * On loading this step is reversed. An annotation file is a plain text format to support 
 * human readability. 
 * The resources used within the plug-in have to be in sync with the workspace, which is 
 * realized through simply using only Eclipse internal file handling techniques.       
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */
public class AnnotationFile implements AnnotationFileHandler {

	/*Constants*/
	public static final char CSVSeperator='';
	public static final char HeaderSeperator='=';
	public static final char ESCAPEChar='@';
	public static final char CSVlineSep='&';
	private String LINESEP=System.getProperty("line.separator");
	private char ECL_FILESEP='/';
	
	/*List of Annotations*/
	private LinkedList annotationItems;
	
	/*Table of header elements*/
	private Hashtable headerElements;
	
	/*relative file path*/
	private String filename=null;
	
	/**
	 * Constructor- should be used if no physical
	 * file exists.
	 */
	public AnnotationFile(){
		annotationItems=new LinkedList();
		headerElements=new Hashtable();
	}
	
	/**
	 * Constuctor- should be used if physical file
	 * exists.
	 * @param file- Filepath relativ to the workbench.
	 */
	public AnnotationFile(String file){		
		filename=file;
		IPath p=new Path(file);
		IFile f=ResourcesPlugin.getWorkspace().getRoot().getFile(p);
		
		if(f.exists()) readFile(file);
		else {
			annotationItems=new LinkedList();
			headerElements=new Hashtable();
		}
	}
	
	/**
	 * Parses a file using the given annotation file 
	 * stucture.
	 * (integer n) - number of header elements
	 * (n* Ini Tags)
	 * (blank line)
	 * (CSV Lines)
	 * @param file- relativ filepath
	 * @return success?
	 */
	public boolean readFile(String file) {
		filename=file;
		/*path expansion -> full qualifaction*/
		String fullFile=ResourcesPlugin.getWorkspace().getRoot().
							getLocation().toString();
		fullFile+=ECL_FILESEP+file;
		
		BufferedReader input=null;
		annotationItems=new LinkedList();
		headerElements=new Hashtable();
		 try {
			input = new BufferedReader(new FileReader(fullFile));
			/*Parse n*/
			String inputLine=input.readLine();
			int i=Integer.parseInt(inputLine);
			/*Parse header items*/
			for(int j=0;j<i;j++){
				inputLine=input.readLine();
				addHeaderItem(inputLine);
			}
			input.readLine();	//skip blank line
			
			/*Parse CSV lines*/
			inputLine=input.readLine();
			while(inputLine!=null){
				AnnotationItem item=new AnnoObject(inputLine);
				addAnnotationItem(item);
				inputLine=input.readLine();				
			}
			
			input.close();
			
		 } 
		 catch (FileNotFoundException e) {
			return false;		//Error on initiation -> break
		 } 
		 catch (IOException e) {
			try {				//Error while reading -> close, break
				input.close();
			} catch (Exception e1) {}
			return false;
		 } 
		 catch (NumberFormatException e){
			try {				//General structure error -> close, break
				input.close();
			} catch (Exception e1) {}
			return false;
		}
		
		return true;
	}

	
	/**
	 * Adds an header item- the input is a key value string
	 * seperated through a HeaderSeperator (default '=').
	 * @param inputLine
	 */
	private void addHeaderItem(String inputLine) {
		
		int sep=inputLine.indexOf(HeaderSeperator);
		if(sep==-1) return;
		addHeaderItem(inputLine.substring(0,sep),inputLine.substring(sep+1));
		
	}

	
	/**
	 * Adds an Item to the header or sets it.
	 * @param key
	 * @param value
	 */
	public void addHeaderItem(String key, String value) {
		if(key==null || value==null) return;
		headerElements.put(key,value);
	}
	
	
	/**
	 * Saves the file- typically used if a file existed before.
	 * @return success?
	 */
	public boolean saveFile() {
		return saveFile(filename);
	}

	/**
	 * Saves the file to the specified location.
	 * @param file- relativ filepath
	 * @return success?
	 */
	public boolean saveFile(String file) {
		String key;
		String value;
		StringBuffer buffer=new StringBuffer();
		
		if(file==null) {
			return false;
		}
		try {
			IPath path=new Path(file);
			IFile f=ResourcesPlugin.getWorkspace().getRoot().getFile(path);
			if(!f.exists()){
				/*Create file- synchronized to workspace*/
				f.create(null,true,null);
			}
			
			/*Compose file content in StringBuffer*/
			Set keys=headerElements.keySet();
			Iterator i=keys.iterator();
			buffer.append(keys.size()+LINESEP);
			/*Header*/
			while(i.hasNext()){
				key=(String)i.next();
				value=(String)headerElements.get(key);
				buffer.append(key+HeaderSeperator+value);
				buffer.append(LINESEP);
			}
			buffer.append(LINESEP);
			i=annotationItems.iterator();
			/*CSV lines*/
			while(i.hasNext()){
				buffer.append(((AnnotationItem)i.next()).getCSVLine());
				buffer.append(LINESEP);
			}
			
			ByteArrayInputStream input=new ByteArrayInputStream(buffer.toString().getBytes());
			
			/*Save- close not necessary*/
			f.setContents(input,true,false,null);
		
			
		} catch (CoreException e) {
			return false;
		} 
		
		return true;
	}


	/**
	 * Adds an annotation.
	 * @param anno
	 */
	public void addAnnotationItem(AnnotationItem anno) {
		if(anno==null) return;
		annotationItems.add(anno);
	}

	
	/**
	 * Removes an annotation.
	 * @param anno
	 * @return
	 */
	public boolean removeAnnotationItem(AnnotationItem anno){
		if (anno==null) return false;
		return annotationItems.remove(anno);
	}

	/**
	 * Returns a list containing all annotations.
	 * @return
	 * 
	 * @uml.property name="annotationItems"
	 */
	public List getAnnotationItems() {
		return annotationItems;
	}

	
	/**
	 * Returns a Hashtable containing the header items.
	 * @return
	 */
	public Hashtable getHeaderItems() {
		return headerElements;
	}

	
	/**
	 * Returns the filename.
	 * @return
	 */
	public String getFileName() {
		return filename;
	}
	
	/**
	 * Returns the projects name.
	 * @return
	 */
	public String getProjectName(){
		int i=filename.indexOf(ECL_FILESEP,1);
		if (i>0)
			return filename.substring(0,i);
		else 
			return filename;
	}

	
	/**
	 * Refreshes the data using the physical file.
	 * @return
	 */
	public boolean refresh() {
		if(filename!=null){
			return readFile(filename);
		}
		return false;
	}

}
