package ssw.annotation.annoview.dialogs;

import java.util.Hashtable;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.texteditor.ITextEditor;

import ssw.annotation.AnnotationPlugin;
import ssw.annotation.AnnotationFileHandler.AnnoObject;
import ssw.annotation.AnnotationFileHandler.AnnotationFile;
import ssw.annotation.Preferences.AnnoPreferencePage;
import ssw.annotation.annoview.AnnotationViewUtils;

/*
 * Created on 07.03.2005
 */

/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 * 
 * This dialog is used to create new annotations and to edit existing ones. There are 
 * three different circumstances when this dialog may appear:
 * 
 * 1. A new annotation is created and there is no annotation file where the annotation could 
 * be appended. In this case a new annotation file has to be created; this is done by 
 * instantiating an AnnotationFile passing the file name. This instance then will create 
 * an in-synch resource and save the information. 
 * 
 * 2. A new annotation is created and a file to append the information exists. In this case 
 * the global data cannot be modified, as it was fixed on file creation. An AnnotationFile 
 * instance is created, passing the file name, causing the instance to load the currently 
 * saved data. After appending the data to the AnnotationFile- list the file may be saved 
 * causing editors and views to refresh themselves. 
 * 
 * 3. An existing annotation is to be edited. This causes the dialog to popup with the 
 * information of the annotation. The information is passed directly to the constructor. 
 * In the resulting dialog the annotation data can be modified. It is not possible to modify 
 * the annotation area but the description and the point modification. Global data cannot 
 * be modified by the resulting dialog.
 * 
 * These modes are distinguished using the calling constructor on the one hand, which enables 
 * the dialog to distinguish between the editor and the new mode. And on the other hand the 
 * fact, that the file exists, which is only questionable in new mode.  
 * New annotation files are created using a fixed default file name. If a file exists 
 * matching this file name an annotation file is supposed to exist. The default file name is 
 * hard coded within this class.
 * 
 * The dialog handles the following data:
 * Authors name
 * Authors email address
 * Maximum score value
 * Score modification value
 * Annotation text
 * 
 * The score values are essential and have to be filled in valid. The other values are optional 
 * from the dialogs point of view. The confirmation of an invalid input causes the dialog to 
 * stay and error messages to pop up. 
 * The data for the created and saved AnnoObject is fetched from the dialog input fields and from 
 * the currently active compilation unit editors highlighted area. From the editor the beginning 
 * line number, the offset and the length are extracted. The offset is the count of signs from the 
 * beginning of the file. The line number is not necessary for the reconstruction of the 
 * highlighted area, but fetched to support human readability of the resulting file. 
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */

public class AnnoInputDialog extends Dialog {
	
	private char ECL_FILESEP='/';
	
	//Input Textfields
	private Text nameTextField;
	private Text emailTextField;
	private Text pointsTextField;
	private Text pointsModiTextField;
	private Text descriptionText;
	//Path to the Java file/Project
	private String javaFile;
	private String project;
	//Default name of an annotation file
	private final String defaltFileName="annotation.ann";
	//Selection Identifiers
	private int selLine;
	private int selCol;
	private int selLength;
	//defines if new data or existing one is handled
	private boolean editorMode=false;
	//annotation file existis?
	private boolean fileExists=false;

	/**
	 * 
	 * @uml.property name="editObj"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	//Editmode vars
	private AnnoObject editObj;

	/**
	 * 
	 * @uml.property name="editFile"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	private AnnotationFile editFile;

	
	/**
	 * Constructor for Editmode.
	 * @param parent
	 * @param ob
	 * @param file
	 */
	public AnnoInputDialog(Shell parent, AnnoObject ob, AnnotationFile file){
		super(parent);
		editorMode=true;
		editObj=ob;
		editFile=file;
	}

	/**
	 * Constructor
	 * @param parent
	 */
	public AnnoInputDialog(Shell parent) {
		super(parent);
	}
	
	/**
	 * Constructor for Collector-/New Mode.
	 * @param parent
	 * @param file
	 * @param project
	 */
	public AnnoInputDialog(Shell parent, String file, String project){
		super(parent);
		javaFile=file;
		this.project=project;
		
		//Fetch the Selection of the java editor
		try{
			IEditorPart editorPart=PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getActivePage().getActiveEditor();
			if(editorPart instanceof ITextEditor){
				ITextEditor editor=(ITextEditor) editorPart;
				ITextSelection selection=(ITextSelection)editor.
					getSelectionProvider().getSelection();
				if(selection.isEmpty()){
					selLine=0;
					selCol=0;
					selLength=0;
				}else{
					selLine=selection.getStartLine();
					selCol=selection.getOffset();
					selLength=selection.getLength();
				}
				
			}else{
				resourceMessage("AI_E_NOTEXTEDIT");
			}
			
		}catch(Exception e){
			resourceMessage("AI_E_NOJAVAEDIT");
		}
	}
	
	/**
	 * Initialisation of the dialog area.
	 */
	protected Control createDialogArea(Composite parent){
		  Composite dialogArea = (Composite)super.createDialogArea(parent);
	      dialogArea.setLayout(new RowLayout(SWT.VERTICAL));
	      
	      //*******General Inforamtion******
	      FillLayout authorLayout=new FillLayout(SWT.VERTICAL);
	      authorLayout.marginHeight=10;
	      authorLayout.marginWidth=10;
	      authorLayout.spacing=3;
	      
	      Group authorInfo=new Group(dialogArea,SWT.FILL);
	      authorInfo.setText(AnnotationPlugin.getResourceString("AI_GENERAL"));
	      authorInfo.setLayout(authorLayout);
	      RowData row=new RowData(350,165);
	      authorInfo.setLayoutData(row);
	      
	      Label nameLabel=new Label(authorInfo,SWT.FILL);
	      nameLabel.setText(AnnotationPlugin.getResourceString("AI_NAME"));
	      nameTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);

	      Label emailLabel=new Label(authorInfo,SWT.FILL);
	      emailLabel.setText(AnnotationPlugin.getResourceString("AI_EMAIL"));
	      emailTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);
	      
	      Label pointsLabel=new Label(authorInfo,SWT.FILL);
	      pointsLabel.setText(AnnotationPlugin.getResourceString("AI_MAXPOINTS"));
	      pointsTextField=new Text(authorInfo,SWT.SINGLE|SWT.BORDER|SWT.FILL);
	      
	      //**********Specific Info************
	      Composite annotationInfo=new Composite(dialogArea,SWT.FILL);
	      annotationInfo.setLayout(new RowLayout(SWT.VERTICAL));	      
	      row=new RowData(350,140);
	      annotationInfo.setLayoutData(row);	      
	      
	      Label pointModi=new Label(annotationInfo,SWT.FILL);
	      pointModi.setText(AnnotationPlugin.getResourceString("AI_POINTMOD"));
	      pointsModiTextField=new Text(annotationInfo,
	      									SWT.SINGLE|SWT.BORDER);
	      row=new RowData(350-5,15);
	      pointsModiTextField.setLayoutData(row);
	      
	      Label desc=new Label(annotationInfo,SWT.FILL);
	      desc.setText(AnnotationPlugin.getResourceString("AI_DESCRIPT"));
	      descriptionText=new Text(annotationInfo,SWT.MULTI|SWT.BORDER|SWT.V_SCROLL|SWT.WRAP);
	      row=new RowData(350-35,65);
	      descriptionText.setLayoutData(row);
	      
	      //Editor or new mode
	      if(editorMode) initEditorMode();
	      else initNewMode();
	      
	      return dialogArea;
	}
	
	/**
	 * Initialize NewMode- if annotation file exists, read general data.
	 * else use defalut data to initialize general data in the
	 * dialog.
	 */
	private void initNewMode() {
		//annotation filepath
		String workSpacePfad=project+ECL_FILESEP+defaltFileName;
	    
		fileExists=ResourcesPlugin.getWorkspace().getRoot().
						getFile(new Path(workSpacePfad)).exists();
		
		//get Plugin's preference Page
		IPreferenceStore pref=AnnotationPlugin.getDefault().getPreferenceStore();
		
		if(pref==null) return;
		
		String name;
		String email;
		String points;
		
		if(fileExists){
						
			//Read general data from existing Annotationfile
			AnnotationFile file=new AnnotationFile(workSpacePfad);
			Hashtable heather=file.getHeaderItems();
			
			name=(String)heather.get(pref.getString(AnnoPreferencePage.D_NAME_KEY));
			email=(String)heather.get(pref.getString(AnnoPreferencePage.D_EMAIL_KEY));
			points=(String)heather.get(pref.getString(AnnoPreferencePage.D_POINTS_KEY));
			nameTextField.setEditable(false);
			emailTextField.setEditable(false);
			pointsTextField.setEditable(false);
							
		}
		else{
			//Read default values
			name=pref.getString(AnnoPreferencePage.D_NAME);
			email=pref.getString(AnnoPreferencePage.D_EMAIL);
			points=String.valueOf(pref.getInt(AnnoPreferencePage.D_POINTS));			
		}

		if(name==null) name="";
		if(email==null) email="";
		if(points==null) points="";
		
		nameTextField.setText(name);
		emailTextField.setText(email);
		pointsTextField.setText(points);	
	}

	/**
	 * Initialize fields in editor mode.
	 */
	private void initEditorMode() {
		Hashtable heather=editFile.getHeaderItems();

		//get Plugin's preference Page
		IPreferenceStore pref=AnnotationPlugin.getDefault().getPreferenceStore();
		
		String name=(String)heather.get(pref.getString(AnnoPreferencePage.D_NAME_KEY));
		String email=(String)heather.get(pref.getString(AnnoPreferencePage.D_EMAIL_KEY));
		String points=(String)heather.get(pref.getString(AnnoPreferencePage.D_POINTS_KEY));
		
		nameTextField.setText(name);
		nameTextField.setEditable(false);
		emailTextField.setText(email);
		emailTextField.setEditable(false);
		pointsTextField.setText(points);
		pointsTextField.setEditable(false);
		pointsModiTextField.setText(String.valueOf(editObj.getPoints()));
		descriptionText.setText(editObj.getDescription());
	}

	/**
	 * Set shell of the Dialog
	 */
	protected void configureShell(Shell newShell) {
	    super.configureShell(newShell);
	    newShell.setText("Annotation");
	 }
	
	/**
	 * Action routine- OK Button pressed.
	 */
	protected void okPressed(){		
		//***********************Editor Mode**************************
		if(editorMode){
			List items=editFile.getAnnotationItems();
			AnnoObject newObj=parseAnnoObject();
			if(newObj==null) {
				resourceMessage("AI_E_INPUT");
				return;
			}
			
			//set non editable data
			newObj.setChecked(editObj.isChecked());
			newObj.setFile(editObj.getFile());
			newObj.setLength(editObj.getLength());
			newObj.setLine(editObj.getLine());
			newObj.setOffset(editObj.getOffset());
			
			if(items.contains(editObj)){
				items.set(items.indexOf(editObj),newObj);
				editFile.saveFile();
			}else{
				resourceMessage("AI_E_EDIT");
			}
		}
		
		//************************Collector/New Mode*************************
		else
		{
			try{
				//check if Integer parsable
				Integer.parseInt(pointsModiTextField.getText());
			}catch(Exception e){
				resourceMessage("AI_E_NOINT");
				return;
			}
		
			AnnoObject obj=parseAnnoObject();
			if(obj==null) return;
		
			String workSpacePfad=project+ECL_FILESEP+defaltFileName;
	    
			AnnotationFile file=new AnnotationFile(workSpacePfad);
			
			//get Plugin's preference Page
			IPreferenceStore pref=AnnotationPlugin.getDefault().getPreferenceStore();
	    
			if(!fileExists){
				file.addHeaderItem(pref.getString(AnnoPreferencePage.D_NAME_KEY),nameTextField.getText());
				file.addHeaderItem(pref.getString(AnnoPreferencePage.D_EMAIL_KEY),emailTextField.getText());
				file.addHeaderItem(pref.getString(AnnoPreferencePage.D_POINTS_KEY),pointsTextField.getText());
			}
	    
			file.addAnnotationItem(obj);
			file.saveFile();

			//Opens/refreshes Annoview with new data
			AnnotationViewUtils.openAnnotationView(file);
		}
		close();
		
	}
	
	/**
	 * Collect the Data of an annotation Item and create an AnnoObject Instance.
	 * @return
	 */
	private AnnoObject parseAnnoObject(){
		String desc=descriptionText.getText();
		
		try{
			int p=Integer.parseInt(pointsModiTextField.getText());
			return new AnnoObject(false, desc,p,javaFile,selLine,selCol,selLength);
		}catch(Exception e){
			resourceMessage("AI_E_NOINT");
			return null;
		}
	}


	/**
	 * Shows a message dialog.
	 * @param message
	 */
	private void showMessage(String message) {
		MessageDialog.openInformation(
				this.getShell(),
				AnnotationPlugin.getResourceString("AI_MESSAGE_TOP"),message);
	}

	/**
	 * Loads a message from resource bundle using key and
	 * shows it via showMessage.
	 * @param key
	 */
	private void resourceMessage(String key){
		showMessage(AnnotationPlugin.getResourceString(key));
	}
}